package tw.net.ocs.example;

import tw.net.ocs.*;
import tw.net.ocs.proxy.*;

/**
	<pre>
	Example 2 shows how to create and run an Agent and a Listener
	on a remote server with the help of a local proxy Agent.

	Compare the code below with that of Example 1 to see how little
	it takes to turn a local application into a client/server
	application with OCSelot.


	----------------------------------
	Scenario:
	HERE           |          THERE
						| Agent+Listener
	----------------------------------

	Start a server in the remote process ("THERE"), as host of Agent
		and Listener,
	start a client in the local process ("HERE"), which communicates
		with this server, and
	create a proxy Agent HERE and pass it a Listener: thereupon the
		proxy Agent
		- binds to the client,
		- creates a real Agent on the server THERE as the proxy's
			subject and passes it the Listener,
		- associates with its remote counter-part so that subsequent
			invocations of the proxy Agent methods are forwarded to
			the remote real Agent.

	To run the demo
		start an OCSServer in a separate shell:
			java -cp ocs.jar tw.net.ocs.OCSServer -start
		and run Example2:
			java -cp ocs.jar;ocs-examples.jar tw.net.ocs.example.Example2
	</pre>

	@author Tillmann Wegst
*/
public class Example2 {
	public static void main(String[] args) {

		System.out.println("Running Example2: Agent+Listener THERE");

			// Create an OCSClient instance that
		OCSClient.createInstance(
			"localhost",	// communicates with a server at this host
			23229,			// and this port,
			null,				// is unnamed, and 
			true				// made the standard client to use by subsequently created proxies
		);
			// Create a proxy Agent and pass it a regular Listener
			// (The proxy Agent constructor allocates a regular Agent on the
			// remote server THERE and passes the Listener on to that.)
		Agent agent = new TwNetOcsExampleAgent(new ListenerImpl());
			// ... and let the Agent THERE act
		agent.act();
	}
}

