package tw.net.ocs.example;

import tw.net.ocs.*;
import tw.net.ocs.proxy.*;

/**
	<pre>
	Example 3a shows how to distribute Agent and Listener
	over two processes: the Agent is run HERE, the Listener THERE.

	As to communication, everything works as in the previous example;
	the point of the example is that a proxy can be used as a
	parameter just like the "real" thing.

	-------------------------------
	Scenario:
	HERE           |          THERE
	Agent          |       Listener
	-------------------------------

	Start a server in a separate process THERE, as host of the
		remote Listener,
	start a client HERE that communicates with this server, and
	create an Agent HERE and pass it a proxy Listener: the proxy Listener
		- binds to the client,
		- creates a real Listener on the server THERE, and
		- associates with its remote counter-part so that whenever the 
			Agent invokes a proxy Listener method it gets forwarded to
			the remote real Listener.

	To run the demo
		start an OCSServer in a separate shell:
			java -cp ocs.jar tw.net.ocs.OCSServer -start
		and run Example3:
			java -cp ocs.jar;ocs-examples.jar tw.net.ocs.example.Example3a
	</pre>

	@author Tillmann Wegst
*/
public class Example3a {
	public static void main(String[] args) {

		System.out.println("Running Example3a: Agent HERE, Listener THERE");

			// Create an OCSClient instance that
		OCSClient.createInstance(
			"localhost",	// communicates with a server at this host
			23229,			// and this port,
			null,				// is unnamed, and 
			true				// made the standard client to use by subsequently created proxies
		);
			// Create an Agent and pass it a proxy Listener
			// (that delegates its invocations to the real Listener
			// which it automatically creates on the remote server)
		Agent agent = new Agent(new TwNetOcsExampleListenerImpl());
			// ... and let the Agent act
		agent.act();
	}
}
